#pragma once
#include <string>
#include <vector>
#include <iosfwd>
#include <functional>

#include "item.h"

#define VERB false

typedef std::wstring Item;

class node {
public:
	node(const Item &key) : key(key), m_left(NULL), m_right(NULL) { }
	
	~node()	{
		delete m_left;
		delete m_right;
	}

	Item getKey() const {
		return key;
	}

	Item setKey(Item k) {
		key = k;
		return key;
	}

	node* left() {
		return m_left;
	}

	node* right() {
		return m_right;
	}

	node* setLeft(node* left) {
		m_left = left;
		return this;
	}

	node* setRight(node* right) {
		m_right = right;
		return this;
	}

	int getSize(void) const {
		return size;
	}

	void setSize(int s) {
		size = s;
	}

	int compare(const node& other)
	{
		if (key < other.key)
			return -1;
		else if (key == other.key)
			return 0;
		else
			return 1;
	}

private:
	node* m_left;
	node* m_right;
	Item key;
	int size;
};

class RBST {
public:
	RBST() : m_root(NULL), m_size(0) {}
	~RBST() { delete m_root; }

	bool add(const Item &key, bool verbose = VERB);
	bool del(const Item &key, bool verbose = VERB);
	bool find(const Item &key, bool verbose = VERB) const;

	node* find(node* from, std::function<int(const Item &k)> f) const;
	node* findFromRoot(std::function<int(const Item &k)> f) const;

	unsigned int size() const { return m_size; }

private:
	node* randomAdd(node* target, node* newNode);
	node* addRoot(node* target, node* newNode);
	node* rightRotate(node* target);
	node* leftRotate(node* target);
	node* del(node* target, Item key);
	node* removeMinLeaf(node* target, node* min);

	node* findMin(node* t) const;
	node* deleteNode(node* target);

	node* find(node* target, Item key) const;

	node* m_root;
	unsigned int m_size;
};

class DataBase
{
public:
	void insert(Item &k)
	{
		tree.add(k);
	}

	Item find(const std::wstring &k) const;
	void findAllLike(const std::wstring &k, std::vector<Item> &result) const;
	void loadFromFile(const std::string &fileName);

private:
	RBST tree;
};
