#include "item.h"

#include <algorithm>
#include <cwctype>
#include <functional>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <locale>
#include <cwchar>

using namespace std;

#define NAMELEN 32
#define STREETLEN 20
#define HOUSELEN 2
#define APARTLEN 2
#define DATELEN 8
#define ITEMLEN 64
#define NUMBERLEN 2

Date::Date()
: mday(1)
, mon(1)
, year(70)
{}

Date::Date(unsigned short d, unsigned short m, unsigned short y)
: mday(d)
, mon(m)
, year(y)
{}

std::wostream& operator<<(std::wostream& out, const Date& d)
{
	out << d.mday << "." << d.mon << "." << d.year;
	return out;
}

namespace {

inline unsigned short bytesToUshort(const wchar_t *l, const wchar_t *h)
{
	return ((*h) << 8) | (*l);
}

std::wstring &rtrim(std::wstring &s)
{
	s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<wint_t, int>(std::iswspace))).base(), s.end());
	return s;
}

}

std::wostream& operator<<(std::wostream &out, const Name &n)
{
	out << n.familyname << " " << n.firstname << " " << n.patronymic;
	return out;
}

Name::Name(const std::wstring &f, const std::wstring &n, const std::wstring &p)
	: familyname(f)
	, firstname(n)
	, patronymic(p)
{
}

namespace item {

Item::Item()
	: house(0)
	, apartament(0)
	, emptyFlag(true)
{
}

void Item::setName(const wchar_t *s, size_t count)
{
	wstring ws(s, count);
	rtrim(ws);

	wstringstream ss(ws);
	ss >> std::noskipws;

	auto fill = [&ss](wstring &s) {
		ss >> s;
		ss.clear();
		wchar_t delim;
		ss >> delim;
	};

	wstring fn;
	wstring n;
	wstring p;
	fill(fn);
	fill(n);
	fill(p);
	name = Name(fn, n, p);
	noEmpty();
}

void Item::setStreet(const wchar_t *s, size_t count)
{
	street = wstring(s, count);
	rtrim(street);
	noEmpty();
}

void Item::setHouse(unsigned short v)
{
	house = v;
	noEmpty();
}

void Item::setApartament(unsigned short v)
{
	apartament = v;
	noEmpty();
}

void Item::setDateOfOccupancy(const Date &d)
{
	dateOfOccupancy = d;
	noEmpty();
}

void Item::setDateOfOccupancy(const wchar_t *s, size_t count)
{
	const size_t numlen = 2;
	unsigned short d = (unsigned short)stoi(wstring(s, numlen));
	unsigned short m = (unsigned short)stoi(wstring(s + numlen, numlen));
	unsigned short y = (unsigned short)stoi(wstring(s + numlen * 2, numlen));
	dateOfOccupancy = Date(d, m, y);
	noEmpty();
}

bool Item::empty() const
{
	return emptyFlag;
}

void Item::noEmpty()
{
	emptyFlag = false;
}

std::wistream& operator>>(wistream& in, Item& item)
{
	wchar_t buff[ITEMLEN+1];
	const wchar_t *cur = buff;
	in.read(buff, ITEMLEN);
	if (in.gcount() < ITEMLEN)
		return in;

	item.noEmpty();
	item.setName(cur, NAMELEN);
	cur += NAMELEN;
	item.setStreet(cur, STREETLEN);
	cur += STREETLEN;
	item.house = bytesToUshort(cur, cur+1);
	cur += HOUSELEN;
	item.apartament = bytesToUshort(cur, cur+1);
	cur += APARTLEN;
	item.setDateOfOccupancy(cur, DATELEN);
	return in;
}

std::wostream& operator<<(std::wostream& out, const Item& item)
{
	if (item.empty()) {
		out << "Item is empty";
	}
	else {
		out
			<< item.name << " "
			<< item.street << " "
			<< item.house << " "
			<< item.apartament << " "
			<< item.dateOfOccupancy;
	}
	return out;
}

}